/*

    Ophcrack is a Lanmanager/NTLM hash cracker based on the faster time-memory
    trade-off using rainbow tables. 
    
    Created with the help of: Maxime Mueller, Luca Wullschleger, Claude
    Hochreutiner and Andreas Huber.

    Copyright 2005 Philippe Oechslin, Cedric Tissieres

    Ophcrack is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Ophcrack is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Ophcrack; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

/* LanManager/NTLM password cracker using a Time-Memory Trade-Off */

static char rcsid[] = "$Id: ophcrack.c,v 2.0 2005/03/24 tissieres Exp $";

/* 
 * $Log: ophcrack.c,v $
 * Revision 2.0  2005/03/24  tissieres
 * Modified for GUI
 *
 * Revision 1.1  2004/09/13 14:26:02  oechslin
 * Fixed bug in -q option
 *
 * Revision 1.0  2004/07/10 19:06:46  oechslin
 * Initial revision
 *
 *
 */



#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#ifndef WIN32
#include <sys/times.h>
#include <sys/mman.h>
#else /* WIN32 */
#include <sys/time.h>
#include <windows.h>
#endif /* WIN32 */
#include <sys/stat.h>

#include "make_hash.h"
#include "make_redux.h"
#include "ophcrack.h"

#ifdef WIN32
#define bzero(p, l) memset(p, 0, l)
#endif /* WIN32 */

/* command line options */
int magic_chains = 0,
  stats = 0,
  parallel = 0,
  verbose = 0,
  batch_mode = 0,
  batch_tables = 1,
  quiet = 0,
  ident_col = 99999,
  cols = 0,
  offset = 0,
  first_table = 0,
  last_table = 20;

int STOP = 0,
    open_tables = 0;

unsigned char 
  lm_hash1[MAX_HASH][8]={{0}}, 
  lm_hash2[MAX_HASH][8]={{0}},
  nt_hash[MAX_HASH][16]={{0}}, 
  empty_hash[8]={0xaa,0xd3,0xb4,0x35,0xb5,0x14,0x04,0xee};

int to_go, hashes=0, n=0, done1[MAX_HASH]={0},done2[MAX_HASH]={0}, userid[MAX_HASH];

char directory[128], info[MAX_HASH][64]={{0}}, password[MAX_HASH][16], 
    password1[MAX_HASH][8]={{0}}, password2[MAX_HASH][8]={{0}}; 

#ifndef WIN32
FILE *startfile[20],*endfile[20],*indexfile[20];
#else
HANDLE startfile[20], endfile[20], indexfile[20];
#endif

/* statistics */
int n_search = 0,
  n_fseek = 0,
  n_hashredux = 0,
  n_false = 0,
  n_false_redux = 0,
  n_match = 0,
  n_loop = 0,
  n_found = 0;

struct tms n_times;
int ticks;
clock_t n_start_time, laps_time; 

#ifdef WIN32
char *
strsep(char** stringp, const char* delim)
{
	char *s;
	const char *spanp;
	int c, sc;
	char *tok;

	if ((s = *stringp) == NULL)
		return (NULL);
	for (tok = s;;) {
		c = *s++;
		spanp = delim;
		do {
			if ((sc = *spanp++) == c) {
				if (c == 0)
					s = NULL;
				else
					s[-1] = 0;
				*stringp = s;
				return (tok);
			}
		} while (sc != 0);
	}
	/* NOTREACHED */
}
 
#endif


/* void usage(char *name)
{
    printf("usage %s [-pvsb -d directory -f first -i row -l last -n tables] -t columns lmhash[:nthash]hash_file_name\n",name);
    printf("      -d directory where tables saved\n");
    printf("      -t columns per table\n");
    printf("      -o table offset (if not equal to t)\n");
    printf("      -i appliy identical redux starting at this column \n");
    printf("      -v verbose mode\n");
    printf("      -s generate statistics\n");
    printf("      -q quiet (no progress indication)\n");
    printf("      -f first table (default 0)\n");
    printf("      -l last table (default 2^31-1)\n");
    printf("      -n number of tables to use at a time\n");
    printf("\n\n");
    printf("  Hashes can be provided in the following hex formats:\n");
    printf("  lmhash\n");
    printf("  lmhash:nthash\n");
    printf("  username:userid:lmhash:nthash:other fieds (as provided by pwdump)\n");
    printf("  or the name of a file containing such hashes, one on each line.\n\n");

    exit(-1);
}
*/

/* void print_stats() {
  clock_t t;

  t = times(&n_times);
  printf("\nStatistics: \n hash-redux calculations: %d\n",
	 n_hashredux);
  printf(" endpoint searched %d\n fseek operations %d\n",
	 n_search, n_fseek);
  printf(" matches found %d\n false alarms %d\n",
	 n_match,
	 n_false);
  printf(" hash-redux operations per false alarms %d\n",
	 (n_false? n_false_redux/n_false: 0));
  printf(" time elapsed %5.2fs\n\n", 
	 (float)(t-n_start_time)/ ticks );
}
*/

void init_stats() {
  n_hashredux=0;
  n_search=0;
  n_fseek=0;
  n_false=0;
  n_false_redux=0;
  n_match=0;
#ifndef WIN32
  n_start_time = times(&n_times);
#else /* WIN32 */
  n_start_time = clock(); 
#endif /* WIN32 */
}

/* turn a 6 char password into an unsigned int */ 
unsigned int binary_pw(char *pw6)
{
  int i,l;
  char *p;
  unsigned int out;

  out=0;
  l=strlen(pw6);
  for (i=0; i<6; i++) {
    p=strchr(chars,pw6[i]);
    if (i< l)
	out=out*37+p-(char*)chars;
    else out*=37;
  }
  return out;
}

/* turn an unsigned int into a 6 char password */
void ascii_pw(char *pw6, unsigned int bin)
{
  int k;
  for (k=5; k>=0; k--) { 
    pw6[k]=chars[bin%37]; 
    bin/=37;
  }
  pw6[6]=0;
}



/* Open all the files  
   =================== */
int open_files() 
{
  unsigned char filename[256];

  int table; 
#ifdef WIN32
  int error;
#endif 

  /* for all available tables */
  for (table=first_table; table<=last_table; table++) {

    /* end points */
    sprintf(filename,"%stable%d.bin",directory,table);
#ifndef WIN32
    if (!(endfile[table]=fopen(filename,"r"))) {
#else
    endfile[table]= CreateFile(filename, GENERIC_READ, 0, NULL, OPEN_EXISTING,
                                FILE_ATTRIBUTE_NORMAL, 0);
    if (GetLastError()) {
#endif
	last_table = table-1;
	break; /* all tables searched */
    }        

    /* starting points */
    sprintf(filename,"%stable%d.start",directory,table);
#ifndef WIN32
    if (!(startfile[table]=fopen(filename,"r")))
#else
    startfile[table]= CreateFile(filename, GENERIC_READ, 0, NULL, OPEN_EXISTING,
                                FILE_ATTRIBUTE_NORMAL, 0);
    if (GetLastError()) 
#endif
      g_print ("cannot open table file");

    /* index table */
    sprintf(filename,"%stable%d.index",directory,table);
#ifndef WIN32
    if (!(indexfile[table]=fopen(filename,"r")))
#else
    indexfile[table]= CreateFile(filename, GENERIC_READ, 0, NULL, OPEN_EXISTING,
                                FILE_ATTRIBUTE_NORMAL, 0);
    if (GetLastError()) 
#endif
      g_print ("cannot open index file");
  }

  if (table != first_table) {
    //if (verbose) printf(" found tables %d through %d\n",first_table, last_table);
    open_tables = 1;
    return 1;
  } else {
    printf(" found no tables!\n");
    return 0;
  }
}

/* Find a end of chain in a table and return the corresponding
   start of chain using three files: index, end and start 
   ================================================================
*/

int binsearch(int table, char *pw, char *startpw)
{
  int i,l,l2;
  unsigned int prefix,postfix,start,offset;
  unsigned short p1;
  int high,low;
  char *p;
#ifdef WIN32
  DWORD dwRead;
#else
  int dwRead;
#endif


  l2 = strlen(pw);

  /*calculate prefix from first 4 bytes */
  prefix=0;
  l=(l2<4 ? l2 : 4); 
  offset=36*36*36*36;
  for (i=0; i<4; i++) {
    p=strchr(chars,pw[i]);
    if (i< l) prefix=prefix*36+p-(char*)chars;
    else {prefix+=offset;offset/=36;}
  }
  if (verbose) 
    g_print("prefix %u\n",prefix);
    
  /*calculate postfix from last 3 bytes */
  postfix=0;
  l=(l2>4 ? l2-4 : 0); offset=36*36*36;
  for (i=0; i<3; i++) {
    p=strchr(chars,pw[i+4]);
    if (i< l) postfix=postfix*36+p-(char*)chars;
    else { postfix+=offset;offset/=36; }
  }

  if (verbose) 
    g_print("postfix %hu\n",postfix);
  
  /* look-up range for this prefix in the index */
#ifndef WIN32
  fseek(indexfile[table],4*prefix,SEEK_SET);
  fread(&low,sizeof(low),1,indexfile[table]);
#else
  SetFilePointer(indexfile[table], 4*prefix, NULL, FILE_BEGIN);
  ReadFile(indexfile[table], &low, sizeof(low), &dwRead, NULL);
#endif
  n_fseek++;
  
  if (low) {
#ifndef WIN32
    fread(&high,sizeof(high),1,indexfile[table]);
#else
    ReadFile(indexfile[table], &high, sizeof(high), &dwRead, NULL);
#endif
    if (high) {
      /* binary search for the postfix in the given range */
      while (1) {
	if (high<=low) return 0;
#ifndef WIN32
        fseek(endfile[table],(high+low)/2*2, SEEK_SET);
        fread(&p1,sizeof(p1),1,endfile[table]);
#else
        SetFilePointer(endfile[table], (high+low)/2*2, NULL,FILE_BEGIN);
        ReadFile(endfile[table], &p1, sizeof(p1), &dwRead, NULL);
#endif
        n_fseek++;

        if(p1==postfix)
          break; /* found! */
	
        if (p1>postfix) 
          { high-=(high-low+1)/2; if (verbose) printf("+");}
        else            
	  { low+=(high-low+1)/2; if (verbose) printf("-");}
      }    
    }
  }
  
  /* read start and change it to ascii: */
#ifndef WIN32
  fseek(startfile[table],(high+low)/2*4,SEEK_SET);
  fread(&start,sizeof(start),1,startfile[table]);
#else
  SetFilePointer(startfile[table], (high+low)/2*4, NULL, FILE_BEGIN);
  ReadFile(startfile[table], &start, sizeof(start), &dwRead, NULL); 
#endif
  n_fseek++;
//  g_print("start = %d", start);

  startpw[6]=0;
  for(i=5;i>=0;i--) {
    startpw[i] = chars[start%36]; 
    start/=36;
  }

  return 1;
}
  
/* Find the password corresponing to HASH in a given table, at a given
   column and starting with a given reduction function
   =================================================== */

int find(unsigned char *hash, char *found, int col, int table)
{
  unsigned char h[8];
  static char pw[8], start[8];
  static int i,l;

  set_redux(col, table);
  
  /* advance to end of chain */

  /* we can save this work if we are in parallel mode, above ident_redux and
   * not in the first table and doing the tree-table optimization... */

  if (!parallel || table==first_table || col < ident_col) {
      make_redux(hash,pw);
      if (verbose) printf ("%d,%s ",n_redux,pw);

      for (l=1; l<cols-col; l++) {
	  make_hash(pw,h);
	  next_redux();
	  make_redux(h,pw);
	  if (verbose) printf("%d,%s ",n_redux,pw);
      }
      n_hashredux+=l;
      if (verbose) printf("%-7.7s: ",pw);
  }
  
  while (gtk_events_pending ()) 
    gtk_main_iteration ();
  
  /* look up the end in the table and get the start */
  if (binsearch(table,pw,start)) {
    set_redux(0, table);
    n_match++;
    /* advance to the colunm that we are exploring */
    for (i=l; i<=cols-1; i++) {
      make_hash(start,h);
      make_redux(h,start); 
      next_redux();
      n_hashredux++;
    }
    make_hash(start,h);
    /* check if we get the same hash: */
    if (!memcmp(&h,hash,sizeof(h))) {
      strncpy(found,start,8);
      return 1;
    }
    else {
      /* false alarm! */
      if (verbose) printf("0");fflush(stdout);
      n_false++;
      n_false_redux += cols-l;
    }
  }
  return 0;
}


/* Read two halfs of LMHash and possibly NTHash and info from string */
int get_hashes(unsigned char lmhash_1[], unsigned char lmhash_2[], 
	       char nthash[], unsigned char *info, int *userid, 
	       char *password1, char *password2, char *password, 
	       int *done1, int *done2, char *h)
{
  /* following formats are supported:
     lmhash
     lmhash:nthash
     username:userid:lmhash:nthash:passwd1:passwd2:passwd (like pwdump2)
  */
  
  unsigned int i,k,l;
  char *p[8] = {0};
  l=0;
  p[l]=strsep(&h,":");
  while (l<8){
    l++;
    if (!(p[l]=strsep(&h,":")) ) break;
  }
  
  if (l==7) {
    if (strcmp(p[0],"")) strncpy(info,p[0],64);
    if (strcmp(p[1],"")) *userid = atoi(p[1]);
    for (i=0; i<8; i++) {
      if (sscanf(p[2]+2*i,"%2x",&k) !=1 ) 
	return 0;
      lmhash_1[i]=(unsigned char)k;
      
      if (sscanf(p[2]+2*i+16,"%2x",&k) !=1 ) 
	return 0;
      lmhash_2[i]=(unsigned char)k;
    }
    
    if (strcmp(p[3],"")){
      for (i=0;i<16;i++)
	if (sscanf(p[3]+2*i,"%2hhx",(unsigned char *)&nthash[i])!=1) 
	  return 0;
    } else 
      nthash[0]=0;
    
    if (strcmp(p[4],"")) {
      strncpy(password1, p[4], strlen(p[4]));
      *done1=1;
    }
    if (strcmp(p[5], "")) {
      strncpy(password2, p[5], strlen(p[5]));
      *done2=1;
    }
    strncpy(password, p[6], strlen(p[6]));
    
  } else {
    
    for (i=0; i<8; i++) {
      if (sscanf(p[0]+2*i,"%2x",&k) !=1 ) 
	return 0;
      lmhash_1[i]=(unsigned char)k;
      
      if (sscanf(p[0]+2*i+16,"%2x",&k) !=1 ) 
	return 0;
      lmhash_2[i]=(unsigned char)k;
    }      
    
    if (l>1) {
      
      for (i=0;i<16;i++)
	if (sscanf(p[1]+2*i,"%2hhx",(unsigned char *)&nthash[i])!=1) 
	  return 0;
    }
  }
  
  return 1;
}

int resolve_nt_hash(char p1[], char p2[], char h[], char pw[])
{
  unsigned int i;
  unsigned int k=0,l;
  unsigned char md4[17],*p;
  
  bzero(pw,15);
  strcpy(pw,p1);
  strcpy(&pw[7],p2);
  
  md4[16]=0;
  
  for (i=0; i<16384; i++) {
    l = k ^ i ^ (i>>1);
    k = i ^ (i>>1);
    p = pw; while ((l=l>>1)) p++;
    *p=(*p>91?toupper(*p):tolower(*p));
    make_nthash(pw,md4);
    
    if (!memcmp(md4,h,16)) 
      return 1;
  }
  return 0;
}


/* Display password and progress indication */
void display(int h, int k)
{
  int i;

  if (h==-1)
    printf("%-32s : %-14s\n","username or lmhash","password");
  if (h>= 0) {
    if (info[h][0])
      printf("%-32s : ",info[h]);
    else {
      for (i=0; i<8; i++) printf("%02hhx",lm_hash1[h][i]);
      for (i=0; i<8; i++) printf("%02hhx",lm_hash2[h][i]);
      printf(" : ");
    }
    if (resolve_nt_hash(password1[h],password2[h],nt_hash[h],password[h]) )
      printf("%-14s    \n",password[h]);
    else
    printf("%-7s%-7s    \n",password1[h],password2[h]);
  }

  if (!quiet) {
#ifndef WIN32
    laps_time = times(&n_times);
#else /* WIN32 */
    laps_time = clock();
#endif /* WIN32 */
    printf("[tables:%d-%d,%2d%% passwords:%d/%d  seconds/pw:%.2f]\r",
	   first_table,
	   last_table,
	   100-(100*k)/cols,
	   n_found, hashes,
	   (n_found?(float)(laps_time - n_start_time)/ticks/n_found:0));
  }
}

int read_file(char* filename) {
  char c, line[200];
  FILE *hash_file;
  
  clear_passwd_arrays();
  
  hashes = 0;  
  n_found = 0;
  
  /* get hash from command line or from file: */
  
  if (!(hash_file=fopen(filename,"r"))) {
    //g_print("%s is neither a valid hash or a filename\n\n\n",filename);
    return 0;
  }
  
  /* Get hashes from file into table */
  fscanf(hash_file,"%199s",line);
  while (!feof(hash_file)) {
    if (get_hashes(lm_hash1[hashes],lm_hash2[hashes],
		    nt_hash[hashes],info[hashes], &userid[hashes], 
		    password1[hashes], password2[hashes], 
		    password[hashes],&done1[hashes], &done2[hashes],line)) 
      //printf("invalid hash found in hash file: %s\n",line);
      //else
      hashes++;
    fscanf(hash_file,"%199s",line);
  }
  fclose(hash_file);
  return hashes;
}


void clear_passwd_arrays(void) {
  int i;
  
  for (i=0; i<hashes; i++) {
    bzero(info[i], sizeof(info[i]));
    bzero(lm_hash1[i], sizeof(lm_hash1[i]));
    bzero(lm_hash2[i], sizeof(lm_hash2[i]));
    bzero(nt_hash[i], sizeof(nt_hash[i]));
    userid[i]=0;
    bzero(password1[i], sizeof(password1[i]));
    bzero(password2[i], sizeof(password2[i]));
    bzero(password[i], sizeof(password[i]));
    done1[i]=0;
    done2[i]=0;
  }  
}


/* MAIN 
   ==== */

int main_cmd()

{
  extern int optind;
  extern char *optarg;
  int h, i, k, t, table, f_t, l_t;
  int display_step;

  /* disabled for GUI  
  while ((c=getopt(argc, argv, "qn:d:vt:o:si:f:l:"))>0) {
    switch ((char)c) {
    case 'd': strcpy(directory, optarg); break;
    case 't': cols = atoi(optarg); break;
    case 'o': offset = atoi(optarg); break;
    case 'i': ident_col = atoi(optarg); break;
    case 'v': verbose=1;break;
    case 's': stats = 1; init_stats(); break;
    case 'f': first_table = atoi(optarg); break;
    case 'l': last_table = atoi(optarg); break;
    case 'n': batch_tables = atoi(optarg); break;
    case 'q': quiet = 1; break;
    case '?': usage(argv[0]);break;
    }
  }
  */

#ifdef WIN32
  strcpy(directory, "10000/");
#else
  snprintf(directory, 127, "%s/10000/", TDIR);
#endif

  if (access(directory, F_OK) == 0) {
    cols = 10000;
    batch_tables = 4;
  }
  else {
#ifdef WIN32
  strcpy(directory, "5000/");
#else
  snprintf(directory, 127, "%s/5000/", TDIR);
#endif
    if (access(directory, F_OK) == 0) {
      cols = 5000;
      offset = 10000;
      batch_tables = 4;
    }
    else {
#ifdef WIN32
      g_print("No tables found!\nPlease put the tables in a directory named either 10000/ or 5000/\n");
#else
      g_print("No tables found!\nPlease put the tables in a directory named either 10000/ or 5000/ in %s\n", TDIR);
#endif
    }
  }
    
  n_found = 0;

  if (!offset) offset=cols;
  init_redux(offset,ident_col);

  /* find out how many ticks we have per second */
#ifndef WIN32
  ticks = sysconf(_SC_CLK_TCK); 
#else /* WIN32 */
  ticks = CLOCKS_PER_SEC; 
#endif /* WIN32 */

  /* open all table files */
  if (!open_tables)
    if (!open_files(directory)) return 0;

  //read_file - GUI
  to_go = 2*hashes;


  //if (!quiet)
  //  display(-1,cols); /* print a header to stdout */

  
  /* Eliminate empty ones: */
  for (i=0; i<hashes; i++) {
    if (!memcmp(lm_hash1[i],empty_hash,
		sizeof(lm_hash1[i]))) {
      strcpy(password1[i],"/EMPTY/");
      done1[i]=1;
      to_go--;
    }
    if (!memcmp(lm_hash2[i],empty_hash,
		sizeof(lm_hash2[i]))) {
      done2[i]=1;
      to_go--;
    }
    if (done1[i] && done2[i])
      { n_found++; fill_tree();}
  }
    
  
  /* search all passwords in all tables: */
  /* ==================================  */

  display_step = cols/100;
  /* for each batch: */
  f_t = first_table; l_t= last_table;
  
  init_stats();
  for (t=f_t; t<= l_t; t+=batch_tables) {
    first_table = t;
    last_table = t+batch_tables-1>l_t? l_t : t+batch_tables-1;
    
    /* for each column of the tables */
    for (k=cols-1; k>=0; k--) {

      /* if (!quiet)
	if (!(k%display_step)) 
	  display (-2,k);
      */
      update_statusbar(k);

      /* for each file containing a table: */
      for (table=first_table; table<=last_table; table++) {
	while (gtk_events_pending ()) {
	  if (STOP)
	    return (0);
	  gtk_main_iteration ();
	}

	/* for each password: */
	for (h=0; h<hashes; h++) {
	  if (!done1[h]) {
	    n_search++;
	    if (find(lm_hash1[h], password1[h], k, table)) {
	      fill_tree();
	      done1[h]=1;
	      if (done2[h]) {n_found++;
	      resolve_nt_hash(password1[h],password2[h],nt_hash[h],password[h]);
	      fill_tree();
	      //display(h,k);
	      update_statusbar(k);}
	      if (!--to_go) break; 
	    }
	  }

	  if (!done2[h]) {
	    n_search++;
	    if (find(lm_hash2[h], password2[h], k, table)) {
	      fill_tree();
	      done2[h]=1; 
	      if (done1[h]) {n_found++; 
	      resolve_nt_hash(password1[h],password2[h],nt_hash[h],password[h]);
	      fill_tree();
	      //display(h,k);
	      update_statusbar(k);}
	      if (!--to_go) break; 
	    }
	  }
	}
	if (!to_go) break;
      }
      if (!to_go) break;
    }
    if (!to_go) break;
  }
  
  /* display hashes that have not been fully cracked:
  for (h=0; h<hashes;h++) 
    if (!done1[h] || !done2[h]){
      if (!done1[h]) strcpy(password1[h],".......");
      if (!done2[h]) strcpy(password2[h],".......");
      display(h,0);
    }
  */
  print_stats();
  return (1); 
}
